/* JSGalBuilder: (c) 2009 by Andreas S. Krebs
 * 
 * This file is part of JSGalBuilder.
 *
 * JSGalBuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSGalBuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSGalBuilder.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sberk.jsgal.datamodel;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import net.sberk.jsgal.datamodel.dist.JSGalDist;
import net.sberk.util.XMLUtil;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

@SuppressWarnings("nls")
public class GalleryCollection
{
	private final static String	GALCOLS_SYSDTD="jsgals.dtd";
	private final static String	DOCTYPE	="jsgals";
	private final static String	DEFAULTGAL		="defaultgal";
	private final static String	DEFAULTIMG		="defaultimg";
	private final static String	HORIZGALSNAV	="horizgalleriesnav";
	private final static String	GALSINHORIZROW	="galleriesinhorizrow";
	private final static String	WRAPGALS		="wrapgalleries";
	private final static String	GALLERY	="gallery";
	private final static String	FILE	="file";
	
	// the collection properties
	private	int	defaultgal=-1;
	private int defaultimg=-1;
	private int horizgalleriesnav=0;
	private int galleriesinhorizrow=5;
	private int wrapgalleries=1;
	
	private ArrayList<GalleryEntry>	gals;	// all the galleries
	
	public GalleryCollection()
	{
		this.gals=new ArrayList<GalleryEntry>();
	}
	
	public int size()
	{
		return gals.size();
	}
	
	public int getDefaultGal()
	{
		return defaultgal;
	}

	public void setDefaultGal(int defaultgal) 
	{
		this.defaultgal = defaultgal;
	}

	public int getDefaultImg()
	{
		return defaultimg;
	}

	public void setDefaultImg(int defaultimg) 
	{
		this.defaultimg = defaultimg;
	}

	public int getHorizGalleriesNav() 
	{
		return horizgalleriesnav;
	}

	public void setHorizGalleriesNav(int horizgalleriesnav)
	{
		this.horizgalleriesnav = horizgalleriesnav;
	}

	public int getGalleriesInHorizRow()
	{
		return galleriesinhorizrow;
	}

	public void setGalleriesInHorizRow(int galleriesinhorizrow)
	{
		this.galleriesinhorizrow = galleriesinhorizrow;
	}

	public int getWrapGalleries()
	{
		return wrapgalleries;
	}

	public void setWrapGalleries(int wrapgalleries)
	{
		this.wrapgalleries = wrapgalleries;
	}

	public int indexOf(GalleryEntry ge)
	{
		return gals.indexOf(ge);
	}
	
	public GalleryEntry getEntryAt(int index)
	{
		return gals.get(index);
	}

	public void addEntry(int index, GalleryEntry ge)
	{
		gals.add(index, ge);
		
	}
	public void addEntry(GalleryEntry ge)
	{
		gals.add(ge);
	}
	
	public void removeEntry(GalleryEntry ge)
	{
		gals.remove(ge);
	}
	
	public void removeEntry(int index)
	{
		gals.remove(index);
	}
	
	public static GalleryCollection parseXML(File f) throws IOException, ParserConfigurationException, SAXException, IllegalArgumentException
	{
		Document	doc=XMLUtil.parseXML(f, true);
		Element		el=doc.getDocumentElement();
		
		if(!DOCTYPE.equals(el.getTagName()))
			throw new IllegalArgumentException("illegal doc type: "+el.getTagName());
		
		GalleryCollection	gc=new GalleryCollection();
		
		gc.defaultgal=ConvUtils.parseInteger(el.getAttribute(DEFAULTGAL), gc.defaultgal);
		gc.defaultimg=ConvUtils.parseInteger(el.getAttribute(DEFAULTIMG), gc.defaultimg);
		gc.horizgalleriesnav=ConvUtils.parseInteger(el.getAttribute(HORIZGALSNAV), gc.horizgalleriesnav);
		gc.galleriesinhorizrow=ConvUtils.parseInteger(el.getAttribute(GALSINHORIZROW), gc.galleriesinhorizrow);
		gc.wrapgalleries=ConvUtils.parseInteger(el.getAttribute(WRAPGALS), gc.wrapgalleries);
		
		NodeList			nl=el.getChildNodes();
		for(int i=0; i<nl.getLength(); i++)
		{
			Node	nd=nl.item(i);
			
			if(nd instanceof Element)
			{
				Element	e=(Element)nd;
				
				if(GALLERY.equals(e.getTagName()))
					gc.addEntry(GalleryEntry.parseDOM(e));
				else
					throw new IllegalArgumentException("illegal tag: "+e.getTagName());
			}
		}
		
		return gc;
	}
	
	public void writeXML(File gcf) throws ParserConfigurationException, TransformerException, IOException
	{
		Document	doc=XMLUtil.createDOM();
	
		doc.appendChild(toDOM(doc));
		
		XMLUtil.writeXML(doc, null, GALCOLS_SYSDTD, new FileOutputStream(gcf));
		
		// now also write the jsgals.dtd
		JSGalDist.writeJSGalsDTD(gcf);
	}
	
	Element toDOM(Document doc)
	{
		Element	el=doc.createElement(DOCTYPE);
		
		el.setAttribute(DEFAULTGAL, ConvUtils.int2String(defaultgal));
		el.setAttribute(DEFAULTIMG, ConvUtils.int2String(defaultimg));
		el.setAttribute(HORIZGALSNAV, ConvUtils.int2String(horizgalleriesnav));
		el.setAttribute(GALSINHORIZROW, ConvUtils.int2String(galleriesinhorizrow));
		el.setAttribute(WRAPGALS, ConvUtils.int2String(wrapgalleries));
		
		for(int i=0; i<gals.size(); i++)
			el.appendChild(gals.get(i).toDOM(doc));
		
		return el;
	}
	
	public static class GalleryEntry
	{
		private String	title;
		private String	file;
		private Object	userData;
		
		public GalleryEntry(String file, String title)
		{
			this.file=file;
			this.title=title;
		}
		
		public String getFile()
		{
			return file;
		}
		
		public void setFile(String file)
		{
			this.file=file;
		}
		
		public String getTitle()
		{
			return title;
		}
		
		public void setTitle(String title)
		{
			this.title=title;
		}
		
		public Object getUserData() 
		{
			return userData;
		}

		public void setUserData(Object userData) 
		{
			this.userData = userData;
		}

		static GalleryEntry parseDOM(Element el)
		{
			return new GalleryEntry(el.getAttribute(FILE), XMLUtil.getElementText(el));
		}
		
		Element toDOM(Document doc)
		{
			Element		el=doc.createElement(GALLERY);
			
			el.setAttribute(FILE, file);
			el.appendChild(doc.createTextNode(title));
			
			return el;
		}
	}
}
