/* JSGalBuilder: (c) 2009 by Andreas S. Krebs
 * 
 * This file is part of JSGalBuilder.
 *
 * JSGalBuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSGalBuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSGalBuilder.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sberk.jsgal.datamodel.dist;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.xml.parsers.ParserConfigurationException;

import net.sberk.jsgal.datamodel.XHTMLPage;
import net.sberk.util.XMLUtil;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;

@SuppressWarnings("nls")
public class JSGalDist
{
	private final static String	INDEX_FILE		="index.html";
	private final static String	JSGAL_DTD_FILE	="jsgal.dtd";
	private final static String	JSGALS_DTD_FILE	="jsgals.dtd";
	
	private final static String	DIST_FILES[]=
	{
		"README",
		"COPYING",
		"jsgal.js",
		"jsgal.css",
		"default.css",
		"classic.css",
		"elegantblack.css",
		"babyblue.css",
		"slickgreen.css",
		"simplegrey.css",
	};
	
	public static void upgradeJSGAL(XHTMLPage xhtmlPage) throws IOException
	{
		createDist(xhtmlPage, true);
	}
	
	public static void generateJSGal(XHTMLPage xhtmlPage) throws IOException
	{
		createDist(xhtmlPage, false);
	}
	
	public static void writeJSGalDTD(File galFile) throws IOException
	{
		createDistFile(galFile.getParentFile(), JSGAL_DTD_FILE, true);
	}

	public static void writeJSGalsDTD(File galColFile) throws IOException
	{
		createDistFile(galColFile.getParentFile(), JSGALS_DTD_FILE, true);
	}
	
	private static void createDist(XHTMLPage xhtmlPage, boolean overwrite)throws IOException
	{
		File	baseFile=xhtmlPage.getXhtmlFile().getParentFile();
		
		// create missing dist files
		for(String s: DIST_FILES)
		{
			createDistFile(baseFile, s, overwrite);
		}
	}

	private static String	distVersion=null;
	private static float 	distVersionNumber;

	public static String getJSGalVersion()
	{
		initDistVersion();
		
		return distVersion;
	}
	
	public static float getJSGalVersionNumber()
	{
		initDistVersion();
		
		return distVersionNumber;		
	}
	
	private static void initDistVersion()
	{
		if(distVersion==null)
		{
			try
			{
				XHTMLPage	xhp=XHTMLPage.parseOnlyUsedTags(getNewIndexDocument());
			
				distVersion=xhp.getJSGalVersion();
				distVersionNumber=xhp.getJSGalVersionNumber();
			}
			catch(Exception e)
			{
				// we ignore this
				distVersion="MISSING: "+e.getMessage();
				distVersionNumber=-1;
			}
		}
	}
	
	public static Document getNewIndexDocument() throws SAXException, ParserConfigurationException, IOException
	{
		return XMLUtil.parseXML(JSGalDist.class.getResourceAsStream(INDEX_FILE), false);
	}
	
	private static void createDistFile(File baseFile, String distFile, boolean overwrite) throws IOException
	{
		File			f=new File(baseFile, distFile);

		// we don't overwrite existing files
		if(overwrite || !f.exists())
		{
			InputStream		is=JSGalDist.class.getResourceAsStream(distFile);
			OutputStream	os=new FileOutputStream(f);
			
			byte			data[]=new byte[1024];
			int				n;
			
			do
			{
				n=is.read(data);
				
				if(n>0)
					os.write(data, 0, n);
			}while(n>=0);
			
			is.close();
			os.close();
		}
	}
}
