/* JSGalBuilder: (c) 2009 by Andreas S. Krebs
 * 
 * This file is part of JSGalBuilder.
 *
 * JSGalBuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSGalBuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSGalBuilder.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sberk.jsgal.gui;

import java.awt.event.ActionEvent;
import java.io.File;
import java.net.URL;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Action;

import net.sberk.jsgal.datamodel.dist.JSGalDist;
import net.sberk.jsgal.gui.I18N.NamedAction;

@SuppressWarnings("serial")
class BuilderActions
{
	JSGalBuilder		builder;
	private ArrayList<Action>	ENABLE_PROJ_LOADED;
	private ArrayList<Action>	ENABLE_PROJ_CLOSED;
	private ArrayList<Action>	ENABLE_MODEL_SELECTED;
	
	protected Action ACTION_MENU_FILE_NEW_PROJECT;
	protected Action ACTION_MENU_FILE_OPEN_PROJECT;
	protected Action ACTION_MENU_FILE_UPGRADE_PROJECT;
	protected Action ACTION_MENU_FILE_CLOSE;
	protected Action ACTION_MENU_FILE_CLOSE_ALL;
	protected Action ACTION_MENU_FILE_SAVE;
	protected Action ACTION_MENU_FILE_SAVE_ALL;
	private	  Action ACTION_MENU_FILE_RECENT_PROJECTS[]; 
	protected Action ACTION_MENU_FILE_EXIT;
	protected Action ACTION_MENU_EDIT_BROWSER;
	protected Action ACTION_MENU_EDIT_PREFS;
	protected Action ACTION_MENU_HELP_CONTENTS;
	protected Action ACTION_MENU_HELP_ABOUT;

	protected BuilderActions(JSGalBuilder jb)
	{
		this.builder=jb;
		
		ENABLE_PROJ_LOADED=new ArrayList<Action>();
		ENABLE_PROJ_CLOSED=new ArrayList<Action>();
		ENABLE_MODEL_SELECTED=new ArrayList<Action>();
		
		ACTION_MENU_FILE_EXIT=new NamedAction(I18N.MENU_FILE_EXIT)
		{ 
			public void actionPerformed(ActionEvent e) { builder.exit(); } 
		};

		ACTION_MENU_HELP_CONTENTS=new NamedAction(I18N.MENU_HELP_CONTENTS)
		{
			public void actionPerformed(ActionEvent e)
			{
				URL url=I18N.getDocURL();
				
				if(url!=null)
					MessageDialog.showMessage(builder, I18N.DOCS_TITLE, url);
				else
					MessageDialog.showError(builder, I18N.MESSAGE_NODOCS, null);
			}
		};
		
		ACTION_MENU_HELP_ABOUT=new NamedAction(I18N.MENU_HELP_ABOUT)
		{
			public void actionPerformed(ActionEvent e)
			{
				String	msg=MessageFormat.format(I18N.ABOUT_MESSAGE, JSGalDist.getJSGalVersion());
				MessageDialog.showMessage(builder, I18N.ICON_BUILDER, I18N.ABOUT_TITLE, msg);
			}
		};
			
		ACTION_MENU_FILE_NEW_PROJECT=new NamedAction(I18N.MENU_FILE_NEW_PROJ, ENABLE_PROJ_CLOSED)
		{ 
			public void actionPerformed(ActionEvent e) { builder.newProject(); } 
		};
			
		ACTION_MENU_FILE_OPEN_PROJECT=new NamedAction(I18N.MENU_FILE_OPEN_PROJ, ENABLE_PROJ_CLOSED)
		{ 
			public void actionPerformed(ActionEvent e) { builder.openProject(); } 
		};

		ACTION_MENU_FILE_UPGRADE_PROJECT=new NamedAction(I18N.MENU_FILE_UPGRADE_PROJ, ENABLE_PROJ_LOADED)
		{ 
			public void actionPerformed(ActionEvent e) { builder.upgradeProject(); } 
		};
		
		ACTION_MENU_FILE_CLOSE=new NamedAction(I18N.MENU_FILE_CLOSE, ENABLE_MODEL_SELECTED)
		{ 
			public void actionPerformed(ActionEvent e) { builder.closeFile(builder.getSelectedModelPanel()); } 
		};		
		ACTION_MENU_FILE_CLOSE_ALL=new NamedAction(I18N.MENU_FILE_CLOSE_ALL, ENABLE_PROJ_LOADED)
		{ 
			public void actionPerformed(ActionEvent e) { builder.closeAll(); } 
		};		

		ACTION_MENU_FILE_SAVE=new NamedAction(I18N.MENU_FILE_SAVE, ENABLE_MODEL_SELECTED)
		{ 
			public void actionPerformed(ActionEvent e) { builder.saveFile(builder.getSelectedModelPanel()); } 
		};

		ACTION_MENU_FILE_SAVE_ALL=new NamedAction(I18N.MENU_FILE_SAVE_ALL, ENABLE_PROJ_LOADED)
		{ 
			public void actionPerformed(ActionEvent e) { builder.saveAll(); } 
		};

		ACTION_MENU_EDIT_BROWSER=new NamedAction(I18N.MENU_EDIT_BROWSER, ENABLE_PROJ_LOADED)
		{ 
			public void actionPerformed(ActionEvent e) { builder.startBrowser(); } 
		};
		
		ACTION_MENU_EDIT_PREFS=new NamedAction(I18N.MENU_EDIT_PREF)
		{ 
			public void actionPerformed(ActionEvent e) { builder.showPreferences(); } 
		};
	}
	
	protected Action[] getRecentProjectActions()
	{
		// first remove old actions from enable list
		if(ACTION_MENU_FILE_RECENT_PROJECTS!=null)
		{
			for(Action a: ACTION_MENU_FILE_RECENT_PROJECTS)
				ENABLE_PROJ_CLOSED.remove(a);
		}
		
		File rf[]=BuilderPrefs.getRecentProjects();
		
		ACTION_MENU_FILE_RECENT_PROJECTS=new Action[rf.length];
		
		for(int i=0; i<rf.length; i++)
		{
			final File	f=rf[i];
			ACTION_MENU_FILE_RECENT_PROJECTS[i]=new NamedAction(f.getName(), ENABLE_PROJ_CLOSED)
			{ 
				public void actionPerformed(ActionEvent e) { builder.loadProject(f); } 
			};			
		}
		
		// fire project updated to make sure actions are enabled correct
		projectUpdated();
		
		return ACTION_MENU_FILE_RECENT_PROJECTS;
	}
	
	protected void addPopupActions(List<Action> alList, final IModelPanel imp)
	{
		alList.add(new NamedAction(I18N.MENU_FILE_SAVE)
			{ 
				public void actionPerformed(ActionEvent e) { builder.saveFile(imp); } 
			});
		alList.add(new NamedAction(I18N.MENU_FILE_CLOSE)
			{ 
				public void actionPerformed(ActionEvent e) { builder.closeFile(imp); } 
			});
		
		if(imp instanceof XHTMLPanel)
		{
			alList.add(null);
			alList.add(ACTION_MENU_EDIT_BROWSER);
		}
	}
	
	protected void projectUpdated()
	{
		boolean	b=(builder.getProjectPage()!=null);
		
		for(Action a: ENABLE_PROJ_LOADED)
			a.setEnabled(b);
		
		b=!b;
		for(Action a: ENABLE_PROJ_CLOSED)
			a.setEnabled(b);
	}
	
	protected void selectionUpdated()
	{
		boolean	b=(builder.getSelectedModelPanel()!=null);
		
		for(Action a: ENABLE_MODEL_SELECTED)
			a.setEnabled(b);
	}
}