/* JSGalBuilder: (c) 2009 by Andreas S. Krebs
 * 
 * This file is part of JSGalBuilder.
 *
 * JSGalBuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSGalBuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSGalBuilder.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sberk.jsgal.gui;

import javax.swing.Action;
import javax.swing.JMenu;
import javax.swing.JMenuBar;

@SuppressWarnings("serial")
class BuilderMenu extends JMenuBar
{
	private JMenu	fileMenu;
	
	protected BuilderMenu(BuilderActions ba)
	{
		super();
		
		// build file menu
		fileMenu=addMenu(I18N.MENU_FILE);
		buildFileMenu(ba);
		
		// build prefs menu
		JMenu	jm=addMenu(I18N.MENU_EDIT);
		jm.add(ba.ACTION_MENU_EDIT_BROWSER);
		jm.addSeparator();
		jm.add(ba.ACTION_MENU_EDIT_PREFS);
		
		// build help menu
		jm=addMenu(I18N.MENU_HELP);
		jm.add(ba.ACTION_MENU_HELP_CONTENTS);
		jm.add(ba.ACTION_MENU_HELP_ABOUT);
	}
	
	protected void updateRecentFiles(BuilderActions ba)
	{
		fileMenu.removeAll();
		buildFileMenu(ba);
	}
	
	private void buildFileMenu(BuilderActions ba)
	{
		// build file menu
		JMenu	jm=fileMenu;
		
		jm.add(ba.ACTION_MENU_FILE_NEW_PROJECT);
		jm.add(ba.ACTION_MENU_FILE_OPEN_PROJECT);
		jm.add(ba.ACTION_MENU_FILE_UPGRADE_PROJECT);
		jm.addSeparator();
		jm.add(ba.ACTION_MENU_FILE_CLOSE);
		jm.add(ba.ACTION_MENU_FILE_CLOSE_ALL);
		jm.addSeparator();
		jm.add(ba.ACTION_MENU_FILE_SAVE);
		jm.add(ba.ACTION_MENU_FILE_SAVE_ALL);
		jm.addSeparator();
		
		// now add recent files
		Action	ra[]=ba.getRecentProjectActions();
		if(ra.length>0)
		{
			for(Action a: ra)
				jm.add(a);
			
			jm.addSeparator();
		}
		
		jm.add(ba.ACTION_MENU_FILE_EXIT);		
	}
	
	private JMenu addMenu(String name[])
	{
		JMenu	jm=new JMenu(name[0]);
		
		if(name.length>1 && name[1]!=null && name[1].length()>0)
		{
			jm.setMnemonic(name[1].charAt(0));
		}
		
		add(jm);
		
		return jm;
	}
}