/* JSGalBuilder: (c) 2009 by Andreas S. Krebs
 * 
 * This file is part of JSGalBuilder.
 *
 * JSGalBuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSGalBuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSGalBuilder.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sberk.jsgal.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.CellEditor;
import javax.swing.Icon;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.ListSelectionModel;
import javax.swing.TransferHandler.DropLocation;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import net.sberk.jsgal.datamodel.Gallery;
import net.sberk.jsgal.datamodel.Gallery.Image;
import net.sberk.jsgal.datamodel.GalleryCollection.GalleryEntry;
import net.sberk.jsgal.gui.I18N.NamedAction;
import net.sberk.jsgal.gui.XHTMLPanel.GalleryEntryUserData;
import net.sberk.util.MiscUtil;

@SuppressWarnings("serial")
class GalleryPanel extends DefaultModelPanel implements TableModel, ListSelectionListener, ActionListener, MouseListener
{
	private Action ACTION_DIALOG_IMG_MOVE_UP;
	private Action ACTION_DIALOG_IMG_MOVE_DOWN;
	private Action ACTION_DIALOG_IMG_ADD;
	private Action ACTION_DIALOG_IMG_REMOVE;

	File			galFile;
	JTable			imageTable;
	private GalleryEntry	galEntry;
	private ArrayList<TableModelListener>	tablstnrs;

	private JTabbedPane	optTabPane;
	private JTextField	titleField;
	private JTextField	imagesDirField;
	private JTextField	thumbsDirField;
	private JComboBox	imgDefScalingBox;
	private JTextField	imageTargetField;
	private	JCheckBox	fixedPageSizeBox;
	private JTextField	pageRowsField;
	private JTextField	pageColsField;
	private JTextField	pagesInRowField;
	private JTextField	thumbWidthField;
	private JTextField	thumbHeightField;
	private JTextField	imageWidthField;
	private JTextField	imageHeightField;
	private JTextField	previewScaleField;
	private JComboBox	previewSelBox;
	private ImagePreviewPanel	previewPanel;
	private ImagePanel			imagePanel;
	
	protected GalleryPanel(JSGalBuilder builder, File galFile, GalleryEntry galEntry, Gallery gal, JTabbedPane parentPane)
	{
		super(builder, gal, parentPane);

		this.galFile=galFile;
		this.galEntry=galEntry;
		
		// create ImageUserData for existing images
		for(int i=gal.size()-1; i>=0; i--)
			new ImageUserData(gal.getImageAt(i));
		
		initActions();
		
		tablstnrs=new ArrayList<TableModelListener>();
		
		setLayout(new BorderLayout(4, 4));

		optTabPane=new JTabbedPane();
		add(optTabPane, BorderLayout.CENTER);
		
		// the general panel
		JPanel	p1=new JPanel();
		p1.setLayout(new BoxLayout(p1, BoxLayout.Y_AXIS));
		
		titleField=SwingHelper.addTextField(p1, I18N.DIALOG_GALLERY_TITLE, 40, gal.getTitle(), this);
		
		
		imagesDirField=SwingHelper.addTextField(p1, I18N.DIALOG_GALLERY_IMAGES_DIR, 20, gal.getImageDir(), this);
		imagesDirField.setEnabled(false);
		imagesDirField.setToolTipText(MiscUtil.getRelativeFile(galFile, gal.getImageDir()).getPath());
		
		thumbsDirField=SwingHelper.addTextField(p1, I18N.DIALOG_GALLERY_THUMBS_DIR, 20, gal.getThumbsDir(), this);
		thumbsDirField.setEnabled(false);
		thumbsDirField.setToolTipText(MiscUtil.getRelativeFile(galFile, gal.getThumbsDir()).getPath());
		
		imgDefScalingBox=SwingHelper.addComboBox(p1, I18N.DIALOG_GALLERY_DEF_IMG_SCALEING, new String[] { I18N.SCALE_VERT, I18N.SCALE_HORIZ, I18N.SCALE_BOTH },
												 scale2Label(gal.getDefScale()), this);

		p1.add(Box.createVerticalStrut(4));
		
		// the thumbs rendering options
		JPanel p2=new JPanel();
		p2.setLayout(new BoxLayout(p2, BoxLayout.Y_AXIS));
		p2.setBorder(BorderFactory.createTitledBorder(I18N.DIALOG_GALLERY_THUMB_RENDERING));
		
		pagesInRowField=SwingHelper.addIntegerField(p2, I18N.DIALOG_GALLERY_THUMB_PAGES_INROW, 4, gal.getPagesInRows(), this);
		
		int	rows=gal.getThumbRows(), cols=gal.getThumbCols();		
		JPanel	p3=new JPanel(new FlowLayout(FlowLayout.LEFT, 4, 2));
		fixedPageSizeBox=SwingHelper.addCheckBox(p3, I18N.DIALOG_GALLERY_THUMB_PAGE_FIXED, (rows>0 || cols>0), this, false);
		pageRowsField=SwingHelper.addIntegerField(p3, null, 3, rows, this, false);
		pageColsField=SwingHelper.addIntegerField(p3, I18N.DIALOG_GALLERY_X, 3, cols, this, false);
		p3.add(new JLabel(I18N.DIALOG_GALLERY_THUMBS));
		p2.add(p3);

		p3=new JPanel(new FlowLayout(FlowLayout.LEFT, 4, 2));
		thumbWidthField=SwingHelper.addIntegerField(p3, I18N.DIALOG_GALLERY_THUMB_SIZE, 4, gal.getThumbWidth(), this, false);
		thumbHeightField=SwingHelper.addIntegerField(p3, I18N.DIALOG_GALLERY_X, 4, gal.getThumbHeight(), this, false);
		p3.add(new JLabel(I18N.DIALOG_GALLERY_PIXEL));
		p2.add(p3);
		
		p1.add(p2);
		p1.add(Box.createVerticalStrut(4));

		// image rendering options
		p2=new JPanel();
		p2.setLayout(new BoxLayout(p2, BoxLayout.Y_AXIS));
		p2.setBorder(BorderFactory.createTitledBorder(I18N.DIALOG_GALLERY_IMAGE_RENDERING));

		p3=new JPanel(new FlowLayout(FlowLayout.LEFT, 4, 2));
		imageWidthField=SwingHelper.addIntegerField(p3, I18N.DIALOG_GALLERY_IMAGE_SIZE, 4, gal.getImageWidth(), this, false);
		imageHeightField=SwingHelper.addIntegerField(p3, I18N.DIALOG_GALLERY_X, 4, gal.getImageHeight(), this, false);
		p3.add(new JLabel(I18N.DIALOG_GALLERY_PIXEL));
		p2.add(p3);
		
		previewScaleField=SwingHelper.addFloatField(p2, I18N.DIALOG_GALLERY_IMAGE_PREVIEW, 6, gal.getPreviewScale(), this);
		imageTargetField=SwingHelper.addTextField(p2, I18N.DIALOG_GALLERY_HREF_TARGET, 20, gal.getTarget(), this);
		
		p1.add(p2);

		p2=new JPanel(new BorderLayout());
		p2.add(p1, BorderLayout.NORTH);
		optTabPane.addTab(I18N.DIALOG_GALLERY_GENERAL, p2);
		
		// the images - first the toolbar
		p1=new JPanel(new BorderLayout(4, 4));
		
		JToolBar tb=new JToolBar(JToolBar.HORIZONTAL);
		tb.add(ACTION_DIALOG_IMG_ADD);
		tb.add(ACTION_DIALOG_IMG_REMOVE);
		tb.addSeparator();
		tb.add(ACTION_DIALOG_IMG_MOVE_UP);
		tb.add(ACTION_DIALOG_IMG_MOVE_DOWN);
		tb.addSeparator();
		previewSelBox=SwingHelper.addComboBox(tb, I18N.DIALOG_GALLERY_PREVIEW_SEL, I18N.DIALOG_GALLERY_PREVIEW_OPTS, 
								I18N.PREVIEW_THUMB, new ActionListener() 
		{

			@Override
			public void actionPerformed(ActionEvent e) 
			{
				togglePreview();
			}
		});
		p1.add(tb, BorderLayout.NORTH);

		p2=new JPanel(new BorderLayout(4, 4));

		// now the table
		p3=new JPanel(new BorderLayout());
		imageTable=new JTable(this);
		imageTable.getSelectionModel().addListSelectionListener(this);
		imageTable.getSelectionModel().setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		imageTable.addMouseListener(this);
		imageTable.setFillsViewportHeight(true);	// we need this for drag and drop into empty area !!!!
		
		// enable drag n drop
		SwingHelper.enableFileDrop(imageTable, new SwingHelper.FileDropListener() 
		{
			@Override
			public boolean canImport(DropLocation location) {	return true;	}	// all locations are valid

			@Override
			public void filesDropped(DropLocation location, File[] files) 
			{
				addImages(files);
			}
		});
		
		p3.add(imageTable.getTableHeader(), BorderLayout.NORTH);
		p3.add(new JScrollPane(imageTable), BorderLayout.CENTER);

		p2.add(p3, BorderLayout.CENTER);
		
		previewPanel=new ImagePreviewPanel(new Dimension(200,150));
		p2.add(previewPanel, BorderLayout.EAST);
		
		p1.add(p2, BorderLayout.CENTER);
		
		imagePanel=new ImagePanel();
		p1.add(imagePanel, BorderLayout.SOUTH);
		
		optTabPane.addTab(I18N.DIALOG_GALLERY_IMAGES, p1);
		
		// setup enablement
		actionOnComponent(fixedPageSizeBox, false);
		
		// finally add it to the tab
		addToTab();
		
		togglePreview();
		selectedImagesChanged();
	}

	private void initActions()
	{
		ACTION_DIALOG_IMG_MOVE_UP=new NamedAction(I18N.DIALOG_GALLERY_MOVE_UP)
		{ 
			public void actionPerformed(ActionEvent e) { moveImagesUp(imageTable.getSelectedRows()); } 
		};
		
		ACTION_DIALOG_IMG_MOVE_DOWN=new NamedAction(I18N.DIALOG_GALLERY_MOVE_DOWN)
		{ 
			public void actionPerformed(ActionEvent e) { moveImagesDown(imageTable.getSelectedRows()); } 
		};
		
		ACTION_DIALOG_IMG_REMOVE=new NamedAction(I18N.DIALOG_GALLERY_REMOVE)
		{ 
			public void actionPerformed(ActionEvent e) { removeImages(imageTable.getSelectedRows()); } 
		};
		
		ACTION_DIALOG_IMG_ADD=new NamedAction(I18N.DIALOG_GALLERY_ADD)
		{ 
			public void actionPerformed(ActionEvent e) { addImages(null); } 
		};		
	}

	void togglePreview()
	{
		Object	sel=previewSelBox.getSelectedItem();
		
		previewPanel.setVisible(!MiscUtil.equals(sel, I18N.PREVIEW_OFF));
		updatePreviewedImage();
	}
	
	private void updatePreviewedImage()
	{
		if(previewPanel.isVisible())
		{
			Object	selObj=getSelectedObject();
			File	previewImg=null;
			String	scale=null;
			
			if(selObj instanceof Image)
			{
				Image			img=(Image)selObj;
				ImageUserData	iud=(ImageUserData)img.getUserData();
			
				if(MiscUtil.equals(previewSelBox.getSelectedItem(), I18N.PREVIEW_IMAGE))
					previewImg=iud.imageFile;
				else
					previewImg=iud.thumbFile;
				
				scale=img.getScale();
			}
			
			previewPanel.setNewImage(previewImg, scale, ImageHelper.ROTATE_0);
		}
	}
	
	protected void addPopupActions(List<Action> alList, final Object obj)
	{
		if(obj instanceof GalleryPanel)
		{
			alList.add(new NamedAction(I18N.DIALOG_GALLERY_ADD)
				{ 
					public void actionPerformed(ActionEvent e) { addImages(null); } 
				});
		}
		else if(obj instanceof Image)
		{
			final Image	img=(Image)obj;
			final int	rows[]=new int[] { getGallery().indexOf(img) };
			
			alList.add(new NamedAction(I18N.DIALOG_GALLERYPOPUP_REMOVE)
				{ 
					public void actionPerformed(ActionEvent e) { removeImages(rows); } 
				});
			alList.add(null);	// seperator

			alList.add(new NamedAction(I18N.DIALOG_GALLERYPOPUP_MOVE_UP)
				{ 
					public void actionPerformed(ActionEvent e) { moveImagesUp(rows); } 
				});
			alList.add(new NamedAction(I18N.DIALOG_GALLERYPOPUP_MOVE_DOWN)
				{ 
					public void actionPerformed(ActionEvent e) { moveImagesDown(rows); } 
				});
		}	
	}
	
	void moveImagesUp(int rows[])
	{
		if(rows!=null && rows.length>0)
		{
			int		lead=imageTable.getSelectionModel().getLeadSelectionIndex();
			Gallery	g=getGallery();
			
			ListSelectionModel	lsm=imageTable.getSelectionModel();
			
			lsm.setValueIsAdjusting(true);
			for(int r: rows)
			{
				if(r>0)
				{
					Image	img=g.getImageAt(r);
					g.removeImage(r);
					
					g.addImage(r-1, img);

					lsm.removeSelectionInterval(r, r);
					lsm.addSelectionInterval(r-1,r-1);
					
					if(r==lead)
						lead--;
				}
			}
			lsm.addSelectionInterval(lead, lead);		// set lead selection
			lsm.setValueIsAdjusting(false);

			fireTableModelEvent(rows[0]-1, rows[rows.length-1], TableModelEvent.UPDATE);
			
			// finally inform tree of changed structure
			builder.treeObjectChanged(this, true);
			selectedImagesChanged();
		}
	}

	void moveImagesDown(int rows[])
	{
		if(rows!=null && rows.length>0)
		{
			int		lead=imageTable.getSelectionModel().getLeadSelectionIndex();
			Gallery	g=getGallery();
			
			ListSelectionModel	lsm=imageTable.getSelectionModel();
			
			lsm.setValueIsAdjusting(true);
			for(int i=rows.length-1; i>=0; i--)
			{
				int	r=rows[i];
				
				if(r<(g.size()-1))
				{
					Image	img=g.getImageAt(r);
					g.removeImage(r);
					
					g.addImage(r+1, img);

					lsm.removeSelectionInterval(r, r);
					lsm.addSelectionInterval(r+1, r+1);
					
					if(r==lead)
						lead++;
				}
			}
			lsm.addSelectionInterval(lead, lead);		// set lead selection
			lsm.setValueIsAdjusting(false);
			
			fireTableModelEvent(rows[0], rows[rows.length-1]+1, TableModelEvent.UPDATE);
			
			// finally inform tree of changed structure
			builder.treeObjectChanged(this, true);
			selectedImagesChanged();
		}	
	}

	void addImages(File images[])
	{
		int	cntOld=getGallery().size();
		
		new ImageImportDialog(builder, this, images);
		
		int	cntNew=getGallery().size();
		if(cntOld!=cntNew)
		{
			fireTableModelEvent(cntOld, cntNew-1, TableModelEvent.INSERT);

			// finally inform tree of changed structure
			builder.treeObjectChanged(this, true);
		}
	}
	
	void removeImages(int rows[])
	{
		if(rows!=null && rows.length>0)
		{
			boolean wasSelected=imageTable.isRowSelected(rows[0]);
			Gallery	g=getGallery();
			
			for(int i=rows.length-1; i>=0; i--)
			{
				int	r=rows[i];
				g.removeImage(r);
			}
			
			fireTableModelEvent(rows[0], rows[rows.length-1], TableModelEvent.DELETE);
			
			// finally inform tree of changed structure
			builder.treeObjectChanged(this, true);
			
			// update selection
			if(wasSelected)
			{
				int	inx=rows[0];
				if(inx>=g.size())
					inx=g.size()-1;
				
				if(inx>=0)
					imageTable.addRowSelectionInterval(inx, inx);
				
				selectedImagesChanged();
			}
		}
	}
	
	public Gallery getGallery()
	{
		return (Gallery)getModel();
	}
	
	public File getGalleryFile()
	{
		return galFile;
	}
	
	public GalleryEntry getGalleryEntry()
	{
		return galEntry;
	}
	
	@Override
	String _getTitle()
	{
		return galFile.getName();
	}
	
	@Override
	public String getTooltip()
	{
		return galFile.getPath();
	}
	
	@Override
	public Icon getIcon()
	{
		return I18N.ICON_GALLERY;
	}

	
	@Override
	public Object getSelectedObject() 
	{
		int	i=imageTable.getSelectionModel().getLeadSelectionIndex();
		
		if(i>=0 && i<getGallery().size())
			return getGallery().getImageAt(i);
		
		return this;
	}

	@Override
	public void setSelectedObject(Object o) 
	{
		int	index=-1;
		if(o instanceof Image)
			index=getGallery().indexOf((Image)o);
		
		if(imageTable.getSelectedRow()!=index)
		{
			if(index<0)
			{
				ListSelectionModel	lsm=imageTable.getSelectionModel();
				
				// take care of clearing the selection - note: clearSelection does not take care of the lead selection :-(
				lsm.setValueIsAdjusting(true);
				lsm.setAnchorSelectionIndex(-1);
				lsm.setLeadSelectionIndex(-1);
				lsm.clearSelection();
				lsm.setValueIsAdjusting(false);
				
				optTabPane.setSelectedIndex(0);		// select general
			}
			else
			{
				imageTable.setRowSelectionInterval(index, index);
				optTabPane.setSelectedIndex(1);		// select images
			}
		}
	}

	private void updateModel()
	{
		Gallery	g=getGallery();
		
		g.setTitle(titleField.getText());
		g.setImageDir(imagesDirField.getText());
		g.setThumbsDir(thumbsDirField.getText());
		g.setDefScale(label2Scale((String)imgDefScalingBox.getSelectedItem()));

		// thumbs options
		g.setPagesInRows(SwingHelper.getIntValue(pagesInRowField));
		g.setThumbRows(SwingHelper.getIntValue(pageRowsField));
		g.setThumbCols(SwingHelper.getIntValue(pageColsField));
		g.setThumbWidth(SwingHelper.getIntValue(thumbWidthField));
		g.setThumbHeight(SwingHelper.getIntValue(thumbHeightField));
		
		// image options
		g.setImageWidth(SwingHelper.getIntValue(imageWidthField));
		g.setImageHeight(SwingHelper.getIntValue(imageHeightField));
		g.setPreviewScale(SwingHelper.getFloatValue(previewScaleField));
		g.setTarget(imageTargetField.getText());
		
		// update current image
		imagePanel.applyChanges();
	}
	
	private String scale2Label(String s)
	{
		if(MiscUtil.equals(s, Gallery.SCALE_HORIZ))
			return I18N.SCALE_HORIZ;
		else if(MiscUtil.equals(s, Gallery.SCALE_BOTH))
			return I18N.SCALE_BOTH;
		
		return I18N.SCALE_VERT;
	}
	
	private String label2Scale(String s)
	{
		if(MiscUtil.equals(s, I18N.SCALE_HORIZ))
			return Gallery.SCALE_HORIZ;
		else if(MiscUtil.equals(s, I18N.SCALE_BOTH))
			return Gallery.SCALE_BOTH;
		
		return Gallery.SCALE_VERT;
	}
	
	String imgScale2Label(String s)
	{
		if(s==null || s.length()<=0)
			return I18N.SCALE_DEFAULT;
		
		return scale2Label(s);
	}
	
	String label2imgScale(String s)
	{
		if(MiscUtil.equals(s, I18N.SCALE_DEFAULT))
			return ""; //$NON-NLS-1$
		
		return label2Scale(s);
	}
	
	@Override
	public void saveUpdatedModel() throws Exception 
	{
		updateModel();
		
		getGallery().writeXML(galFile);
		
		setModified(false);
	}
	
	@Override
	public boolean closePanel() 
	{
		if(isModified())
		{
			Boolean	b=MessageDialog.getSaveModifiedFile(this, galFile);
			
			if(b==null)
				return false;
			
			if(b.booleanValue())
			{
				try
				{
					saveUpdatedModel();
				}
				catch(Exception e)
				{
					builder.reportProblem(e);
					return false;
				}
			}
		}
		
		builder.removePanel(this);
		
		GalleryEntryUserData	gud=(GalleryEntryUserData)galEntry.getUserData();
		gud.getXHTMLPanel().openGalleryClosed(galEntry);
		
		return true;
	}

	@Override
	public void showPopup(Component source, Object obj, int x, int y) 
	{
		ArrayList<Action>	al=new ArrayList<Action>();
		
		if(obj instanceof IModelPanel)
		{
			builder.getBuilderActions().addPopupActions(al, (IModelPanel)obj);
			al.add(null);	// seperator
		}

		addPopupActions(al, obj);
		
		SwingHelper.showPopup(source, x, y, al);
	}
	
	private void selectedImagesChanged()
	{
		Gallery	g=getGallery();
		int		min=imageTable.getSelectionModel().getMinSelectionIndex();
		int		max=imageTable.getSelectionModel().getMaxSelectionIndex();

		ACTION_DIALOG_IMG_MOVE_UP.setEnabled(min>0);
		ACTION_DIALOG_IMG_MOVE_DOWN.setEnabled(max>=0 && max<(g.size()-1));
		ACTION_DIALOG_IMG_REMOVE.setEnabled(min>=0 && max<g.size());
		
		imagePanel.updateSelectedImage();
		updatePreviewedImage();
		builder.setSelectedObject(this, getSelectedObject());
	}
	
	// the stuff for the actions we provide
	// the stuff for the galleries collection table model
	@Override
	public void addTableModelListener(TableModelListener l) 
	{
		tablstnrs.add(l);
	}

	@Override
	public void removeTableModelListener(TableModelListener l) 
	{
		tablstnrs.remove(l);
	}

	void fireTableModelEvent(int firstRow, int lastRow, int type)
	{
		TableModelEvent	e=new TableModelEvent(this, firstRow, lastRow, TableModelEvent.ALL_COLUMNS, type);
		
		for(TableModelListener l: tablstnrs)
			l.tableChanged(e);
		
		setModified(true);
	}
	
	@Override
	public Class<?> getColumnClass(int columnIndex) 
	{
		return String.class;
	}

	@Override
	public int getColumnCount() 
	{
		return 3;
	}

	@Override
	public String getColumnName(int columnIndex) 
	{
		if(columnIndex==0)
			return I18N.COLUMN_IMAGE_FILE;
		else if(columnIndex==1)
			return I18N.COLUMN_IMAGE_NAME;
		else if(columnIndex==2)
			return I18N.COLUMN_IMAGE_DESCRIPTION;
		
		return null;
	}

	@Override
	public int getRowCount() 
	{
		return getGallery().size();
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) 
	{
		Gallery			g=getGallery();
		Image			img=g.getImageAt(rowIndex);
		
		if(columnIndex==0)
			return img.getFile();
		else if(columnIndex==1)
			return img.getName();
		else if(columnIndex==2)
			return img.getDescr();

		return null;
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) 
	{
		return (columnIndex>=1);	// file is not editable
	}

	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex) 
	{
		Image	img=getGallery().getImageAt(rowIndex);
		
		if(columnIndex==1)
		{
			img.setName((String)value);
		
			setModified(true);
			
			imagePanel.nameUpdated(img);
		}
		else if(columnIndex==2)
		{
			img.setDescr((String)value);
		
			setModified(true);
			
			imagePanel.descrUpdated(img);
		}
	}
	
	// stuff for listSelectionListener
	@Override
	public void valueChanged(ListSelectionEvent e) 
	{
		if(!e.getValueIsAdjusting())
			selectedImagesChanged();
	}

	
	// stuff for ActionListener
	@Override
	public void actionPerformed(ActionEvent e) 
	{	
		if(e.getSource() instanceof Component)
			actionOnComponent((Component)e.getSource(), true);
		
		setModified(true);
	}	
	
	private void actionOnComponent(Component box, boolean validateValues)
	{
		if(box==fixedPageSizeBox)
		{
			boolean b=fixedPageSizeBox.isSelected();
			
			pageRowsField.setEnabled(b);
			pageColsField.setEnabled(b);
			
			if(!b && validateValues)
			{
				pageRowsField.setText("0"); //$NON-NLS-1$
				pageColsField.setText("0"); //$NON-NLS-1$
			}			
		}
	}
	
	// stuff for mouse listener
	@Override
	public void mouseClicked(MouseEvent e)
	{
		// we ignore this
	}

	@Override
	public void mouseEntered(MouseEvent e)
	{
		// we ignore this
	}

	@Override
	public void mouseExited(MouseEvent e)
	{
		// we ignore this
	}

	@Override
	public void mousePressed(MouseEvent e)
	{
		checkForPopup(e);
	}

	@Override
	public void mouseReleased(MouseEvent e)	
	{
		checkForPopup(e);
	}

	private void checkForPopup(MouseEvent e)
	{
		if(e.isPopupTrigger() && e.getSource()==imageTable)
		{
			Point p = new Point(e.getX(), e.getY());
			
			int row = imageTable.rowAtPoint(p);
			if(row>=0 && row<getGallery().size())
			{
				CellEditor	ce=imageTable.getCellEditor();
				if(ce!=null)
					ce.stopCellEditing();
				
				showPopup(imageTable, getGallery().getImageAt(row), e.getX(), e.getY());
			}
		}
	}

	public class ImageUserData
	{
		File			imageFile;
		String			imagePath;
		File			thumbFile;
		String			thumbPath;
		
		ImageUserData(Image img)
		{
			Gallery		g=getGallery();
			
			imageFile=MiscUtil.getRelativeFile(galFile, g.getImageDir()+'/'+img.getFile());
			thumbFile=MiscUtil.getRelativeFile(galFile, g.getThumbsDir()+'/'+img.getThumb());

			try
			{
				imagePath=imageFile.getCanonicalPath();
			}
			catch(IOException e)
			{
				imagePath=imageFile.getPath();
			}
			
			try
			{
				thumbPath=thumbFile.getCanonicalPath();
			}
			catch(IOException e)
			{
				thumbPath=thumbFile.getPath();
			}
			
			img.setUserData(this);
		}
		
		protected String getImagePath()
		{
			return imagePath;
		}
		
		protected String getThumbPath()
		{
			return thumbPath;
		}
		
		protected GalleryPanel getGalleryPanel()
		{
			return GalleryPanel.this;
		}
	}
	
	class ImagePanel extends JPanel implements DocumentListener, ActionListener
	{
		private	Image	selImg;
		
		private JTextField	nameField;
		private JTextField	descrField;
		private JComboBox	scaleBox;
		private JTextField	fileField;
		private JTextField	thumbField;
		
		ImagePanel()
		{
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			setBorder(BorderFactory.createTitledBorder(I18N.DIALOG_GALIMG_DETAILS));

			nameField=SwingHelper.addTextField(this, I18N.DIALOG_GALIMG_NAME, 20, null, this);
			nameField.setEnabled(false);
			descrField=SwingHelper.addTextField(this, I18N.DIALOG_GALIMG_DESCR, 40, null, this);
			descrField.setEnabled(false);
			
			scaleBox=SwingHelper.addComboBox(this, I18N.DIALOG_GALIMG_SCALEING, new String[] { I18N.SCALE_DEFAULT, I18N.SCALE_VERT, I18N.SCALE_HORIZ, I18N.SCALE_BOTH },
					 null, this);
			scaleBox.setEnabled(false);
			
			fileField=SwingHelper.addTextField(this, I18N.DIALOG_GALIMG_FILE, 20, null);
			fileField.setEnabled(false);
			thumbField=SwingHelper.addTextField(this, I18N.DIALOG_GALIMG_THUMB, 20, null);
			thumbField.setEnabled(false);
			
			updateSelectedImage();
		}
		
		void applyChanges()
		{
			if(selImg!=null)
			{
				boolean changed=false;
				
				if(!MiscUtil.equals(selImg.getName(), nameField.getText()))
				{
					selImg.setName(nameField.getText());
					changed=true;
				}
				
				if(!MiscUtil.equals(selImg.getDescr(), descrField.getText()))
				{
					selImg.setDescr(descrField.getText());
					changed=true;
				}
				
				String	scale=label2imgScale((String)scaleBox.getSelectedItem());
				if(!MiscUtil.equals(selImg.getScale(), scale))
				{
					selImg.setScale(scale);
					changed=true;
				}
				
				if(changed)
				{
					int	row=getGallery().indexOf(selImg);
					fireTableModelEvent(row, row, TableModelEvent.UPDATE);
				}
			}
		}
		
		void nameUpdated(Image img)
		{
			if(img==selImg)	// only update if really same image
			{
				nameField.setText(selImg.getName());
			}
		}
		
		void descrUpdated(Image img)
		{
			if(img==selImg)	// only update if really same image
			{
				descrField.setText(selImg.getDescr());
			}
		}
		
		private boolean updating=false;
		void updateSelectedImage()
		{
			updating=true;
			
			Object	obj=getSelectedObject();
			
			Image	newImg=(obj instanceof Image) ? (Image)obj : null;
			if(newImg!=selImg)
			{
				applyChanges();
				
				selImg=newImg;
			}
			
			if(selImg==null)
			{
				nameField.setText(null);
				nameField.setEnabled(false);
				descrField.setText(null);
				descrField.setEnabled(false);
				scaleBox.setSelectedItem(null);
				scaleBox.setEnabled(false);
				
				fileField.setText(null);
				fileField.setToolTipText(null);
				thumbField.setText(null);
				thumbField.setToolTipText(null);
			}
			else
			{
				nameField.setText(selImg.getName());
				nameField.setEnabled(true);
				descrField.setText(selImg.getDescr());
				descrField.setEnabled(true);
				scaleBox.setSelectedItem(imgScale2Label(selImg.getScale()));
				scaleBox.setEnabled(true);
				
				ImageUserData	iud=(ImageUserData)selImg.getUserData();
				fileField.setText(selImg.getFile());
				fileField.setToolTipText(iud.getImagePath());
				thumbField.setText(selImg.getThumb());
				thumbField.setToolTipText(iud.getThumbPath());		
			}
			
			updating=false;
		}

		@Override
		public void changedUpdate(DocumentEvent e) 
		{
			dataModified();
		}

		@Override
		public void insertUpdate(DocumentEvent e)
		{
			dataModified();
		}

		@Override
		public void removeUpdate(DocumentEvent e)
		{
			dataModified();
		}
		
		private void dataModified()
		{
			if(!updating)
			{
				setModified(true);
				applyChanges();
			}
		}

		@Override
		public void actionPerformed(ActionEvent e) 
		{
			dataModified();
		}
	}
}