/* JSGalBuilder: (c) 2009 by Andreas S. Krebs
 * 
 * This file is part of JSGalBuilder.
 *
 * JSGalBuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSGalBuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSGalBuilder.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sberk.jsgal.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.net.URL;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListCellRenderer;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.border.EmptyBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLDocument;

import net.sberk.jsgal.gui.I18N.NamedAction;
import net.sberk.util.MiscUtil;

@SuppressWarnings("serial")
public class MessageDialog extends JDialog
{
	private ActionOption	performedAction=null;
	JButton		buttons[];
	JPanel		msgPanel;
	JPanel		buttonPanel;
	
	private MessageDialog(Dialog owner)
	{
		super(owner, true);
	}
	
	private MessageDialog(Frame owner)
	{
		super(owner, true);
	}
	
	void optionActionPerformed(ActionOption ao)
	{
		this.performedAction=ao;
		
		setVisible(false);
		dispose();
	}
	
	@SuppressWarnings("nls")
	private void initDialog(String title, String msg, URL url, Icon icon, String opts[][], int initalOpt)
	{
		setTitle(title);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	
		boolean	isHTML=false;
		String	htmlMsg=null;
		if(url!=null)
			isHTML=true;
		else if(msg!=null && msg.startsWith("<html>"))
		{
			isHTML=true;
			htmlMsg=msg;
		}
		else if(msg!=null && msg.indexOf('\n')>=0)
		{
			htmlMsg="<html>"+msg.replace("\n", "<br>")+"</html>";	// create line multiline label
		}
		else
			htmlMsg=msg;

		JPanel	cp=new JPanel(new BorderLayout(6,4));
		cp.setBorder(new EmptyBorder(4, 4, 4, 4));
		setContentPane(cp);
		
		msgPanel=new JPanel(new BorderLayout(4, 4));
		if(htmlMsg!=null || url!=null)
		{
			JComponent	lcomp;
			
			if(isHTML)
			{
				JEditorPane	jep;
				
				if(url!=null)
				{
					try
					{
						jep=new JEditorPane();
						
						// create a custom document, to prevent async loading issue with window pack !!!
						HTMLDocument	doc=new HTMLDocument()
						{
							@Override
							public int getAsynchronousLoadPriority() { return -1; }
						};				
						
						jep.setDocument(doc);
						jep.setPage(url);						
					}
					catch(Exception e)
					{
						jep=new JEditorPane("text/html", "<html><body><pre>"+MiscUtil.getStacktrace(e)+"</pre></body></html>");
					}
					
					lcomp=new JScrollPane(jep);
				}
				else
				{
					jep=new JEditorPane("text/html", htmlMsg);
					jep.setOpaque(false);
					
					lcomp=jep;
				}
				
				jep.setEditable(false);  
				// make sure we use the normal UI font !!! 
				jep.putClientProperty(JEditorPane.HONOR_DISPLAY_PROPERTIES, Boolean.TRUE);
				jep.addHyperlinkListener(new HyperlinkListener()
				{  
					public void hyperlinkUpdate(HyperlinkEvent hle) 
					{  
						if(HyperlinkEvent.EventType.ACTIVATED.equals(hle.getEventType()))
						{
							try
							{
								BuilderPrefs.startBrowser(hle.getURL().toURI());
							}
							catch(Exception e)
							{
								e.printStackTrace();
							}
						}
					}
				});

			}
			else
				lcomp=new JLabel(htmlMsg);
			
			msgPanel.add(lcomp, BorderLayout.CENTER);
		}
		
		if(icon!=null)
		{
			JPanel	p=new JPanel();
			p.add(new JLabel(icon));
			msgPanel.add(p, BorderLayout.WEST);
		}
		
		getContentPane().add(msgPanel, BorderLayout.CENTER);

		buttonPanel=new JPanel(new BorderLayout(4, 4));
		getContentPane().add(buttonPanel, BorderLayout.SOUTH);
		JPanel	p=new JPanel();
		buttonPanel.add(p, BorderLayout.CENTER);
		
		buttons=new JButton[opts.length];
		for(int i=0; i<opts.length; i++)
		{
			Action	a=new ActionOption(i, opts[i]);
			
			buttons[i]=new JButton(a);
			p.add(buttons[i]);
			
			if(i==initalOpt)
				buttons[i].requestFocusInWindow();
		}
		
		pack();
		setLocationRelativeTo(getParent());
	}

	public static void showError(Component parent, String msg, String details)
	{
		MessageDialog	md=createMessageDialogForParent(parent);
		
		md.initDialog(I18N.MESSAGE_TITLE_ERROR, msg, null, I18N.ICON_DIALOG_ERROR, new String[][] { I18N.MESSAGE_OPT_OK }, 0);
		
		if(details!=null)
			md.new DetailsPanel(details);
		
		md.setVisible(true);
	}

	public static void showWarning(Component parent, String msg, String details)
	{
		MessageDialog	md=createMessageDialogForParent(parent);
		
		md.initDialog(I18N.MESSAGE_TITLE_WARNING, msg, null, I18N.ICON_DIALOG_WARN, new String[][] { I18N.MESSAGE_OPT_OK }, 0);

		if(details!=null)
			md.new DetailsPanel(details);
		
		md.setVisible(true);
	}

	public static boolean showYesNoWarning(Component parent, String msg)
	{
		MessageDialog	md=createMessageDialogForParent(parent);
		
		md.initDialog(I18N.MESSAGE_TITLE_WARNING, msg, null, I18N.ICON_DIALOG_WARN, new String[][] { I18N.MESSAGE_OPT_YES, I18N.MESSAGE_OPT_NO }, 0);

		md.setVisible(true);
		
		return (md.performedAction!=null && md.performedAction.getIndex()==0);
	}

	public static boolean showYesNoConfirm(Component parent, String msg)
	{
		MessageDialog	md=createMessageDialogForParent(parent);
		
		md.initDialog(I18N.MESSAGE_TITLE_CONFIRM, msg, null, I18N.ICON_DIALOG_QUESTION, new String[][] { I18N.MESSAGE_OPT_YES, I18N.MESSAGE_OPT_NO }, 0);

		md.setVisible(true);
		
		return (md.performedAction!=null && md.performedAction.getIndex()==0);
	}
	
	public static void showInfo(Component parent, String msg)
	{
		MessageDialog	md=createMessageDialogForParent(parent);
		
		md.initDialog(I18N.MESSAGE_TITLE_INFO, msg, null, I18N.ICON_DIALOG_INFO, new String[][] { I18N.MESSAGE_OPT_OK }, 0);
		
		md.setVisible(true);
	}

	public static void showInfo(Component parent, String title, String msg)
	{
		MessageDialog	md=createMessageDialogForParent(parent);
		
		md.initDialog(title, msg, null, I18N.ICON_DIALOG_INFO, new String[][] { I18N.MESSAGE_OPT_OK }, 0);
		
		md.setVisible(true);
	}

	public static void showMessage(Component parent, Icon icon, String title, String msg)
	{
		MessageDialog	md=createMessageDialogForParent(parent);
		
		md.initDialog(title, msg, null, icon, new String[][] { I18N.MESSAGE_OPT_OK }, 0);
		
		md.setVisible(true);
	}
	
	public static void showMessage(Component parent, String title, URL url)
	{
		MessageDialog	md=createMessageDialogForParent(parent);
		
		md.initDialog(title, null, url, null, new String[][] { I18N.MESSAGE_OPT_OK }, 0);
		
		// make sure we have a resonable size - in case URL was not yet loaded
		if(md.getWidth()<400 || md.getHeight()<300)
		{
			md.setSize(400, 300);
			md.setLocationRelativeTo(parent);
		}
		
		md.setVisible(true);
	}
	
	public static String getInput(Component parent, String title, String msg)
	{
		return getInput(parent, title, msg, null, null);
	}

	public static String getInput(Component parent, String title, String msg, String value)
	{
		return getInput(parent, title, msg, value, null);
	}
	
	public static String getInput(Component parent, String title, String msg, String value, Component addComp)
	{
		MessageDialog	md=createMessageDialogForParent(parent);
		
		md.initDialog(title, null, null, I18N.ICON_DIALOG_QUESTION, new String[][] { I18N.MESSAGE_OPT_OK, I18N.MESSAGE_OPT_CANCEL }, -1);
		
		InputPanel	ip=md.new InputPanel(msg, value, addComp);
				
		md.setVisible(true);
		
		if(md.performedAction!=null && md.performedAction.getIndex()==0)
			return ip.getText();

		return null;
	}
	
	public static Map<File, Boolean> getSaveModifiedFiles(Component parent, File files[])
	{
		MessageDialog	md=createMessageDialogForParent(parent);
		md.initDialog(I18N.MESSAGE_TITLE_SAVE_CHANGES, null, null, I18N.ICON_DIALOG_QUESTION, new String[][] { I18N.MESSAGE_OPT_OK, I18N.MESSAGE_OPT_CANCEL }, -1);
				
		SelectFilesPanel	sfp=md.new SelectFilesPanel(files);

		md.setVisible(true);
		
		if(md.performedAction==null || md.performedAction.getIndex()==1)	// cancel ?
			return null;
		
		boolean					selFiles[]=sfp.getFilesSelection();
		HashMap<File, Boolean>	map=new HashMap<File, Boolean>();
		for(int i=0; i<files.length; i++)
		{
			map.put(files[i], new Boolean(selFiles[i]));
		}
		
		return map;
	}

	public static Boolean getSaveModifiedFile(Component parent, File file)
	{
		MessageDialog	md=createMessageDialogForParent(parent);
		String			msg=MessageFormat.format(I18N.MESSAGE_MSG_SAVE_CHANGES, file.getName());
		md.initDialog(I18N.MESSAGE_TITLE_SAVE_CHANGES, msg, null, I18N.ICON_DIALOG_QUESTION, new String[][] { I18N.MESSAGE_OPT_YES, I18N.MESSAGE_OPT_NO, I18N.MESSAGE_OPT_CANCEL }, -1);
	
		md.setVisible(true);
		
		if(md.performedAction==null || md.performedAction.getIndex()==2)
			return null;
		
		return new Boolean(md.performedAction.getIndex()==0);
	}
	
	private static MessageDialog createMessageDialogForParent(Component parent)
	{
		if(parent instanceof Frame)
			return new MessageDialog((Frame)parent);
		else if(parent instanceof Dialog)
			return new MessageDialog((Dialog)parent);
		else if(parent!=null)
			return createMessageDialogForParent(parent.getParent());
		
		return new MessageDialog((Dialog)null);
	}
	
	class ActionOption extends NamedAction
	{
		private int	index;
		
		ActionOption(int index, String names[])
		{
			super(names);
			
			this.index=index;
		}
		
		ActionOption(int index, String name)
		{
			super(name);
			
			this.index=index;
		}

		int getIndex()
		{
			return index;
		}
		
		@Override
		public void actionPerformed(ActionEvent e) 
		{
			optionActionPerformed(this);
		}
	}
	
	class DetailsPanel extends JScrollPane
	{
		private JButton		showHideButton;
		private Action		ACTION_SHOW_DETAILS;
		private Action		ACTION_HIDE_DETAILS;
		
		DetailsPanel(String details)
		{
			ACTION_SHOW_DETAILS=new NamedAction(I18N.MESSAGE_SHOW_DETAILS)
			{
				@Override
				public void actionPerformed(ActionEvent e) 
				{
					toggleDetails();
				}
			};

			ACTION_HIDE_DETAILS=new NamedAction(I18N.MESSAGE_HIDE_DETAILS)
			{
				@Override
				public void actionPerformed(ActionEvent e) 
				{
					toggleDetails();
				}
			};
			
			JPanel	p=new JPanel();
			showHideButton=new JButton(ACTION_SHOW_DETAILS);
			p.add(showHideButton);
			buttonPanel.add(p, BorderLayout.EAST);
			
			String	lines[]=details.split("\r\n|\r|\n"); //$NON-NLS-1$
			int	rows=lines.length;
			int	cols=0;
			
			for(String s: lines)
			{
				if(s.length()>cols)
					cols=s.length();
			}
			
			// set min/max
			if(rows<5)	rows=5;
			if(rows>20)	rows=20;
			if(cols<40)	cols=40;
			if(cols>100)cols=100;

			JTextArea	jta=new JTextArea(details, rows, cols);
			jta.setOpaque(false);
			setOpaque(false);
			
			setViewportView(jta);
			setBorder(BorderFactory.createTitledBorder(I18N.MESSAGE_DETAILS));
			setVisible(false);
			
			msgPanel.add(this, BorderLayout.SOUTH);
			
			validate();		
			MessageDialog.this.pack();	
		}
		
		void toggleDetails()
		{
			boolean	show=!isVisible();
			Action	a=show ? ACTION_HIDE_DETAILS : ACTION_SHOW_DETAILS;
			
			showHideButton.setAction(a);
			setVisible(show);
			
			validate();
			MessageDialog.this.pack();	
		}
	}
	
	class SelectFilesPanel extends JPanel implements ListCellRenderer
	{
		File		files[];
		boolean		selFiles[];
		JPanel		cellComp;
		JCheckBox	checkBox;
		private DefaultListCellRenderer	defRenderer;
		
		SelectFilesPanel(File files[])
		{
			super(new BorderLayout(4, 4));
		
			this.files=files;
			this.selFiles=new boolean[files.length];
			
			// setup cell renderer 
			cellComp=new JPanel(new BorderLayout(4,4));
			checkBox=new JCheckBox();
			
			cellComp.add(checkBox, BorderLayout.WEST);
			defRenderer=new DefaultListCellRenderer();
			cellComp.add(defRenderer, BorderLayout.CENTER);

			// setup list
			add(new JLabel(I18N.MESSAGE_MSG_SELECT_SAVE_CHANGES, JLabel.LEFT), BorderLayout.NORTH);
			
			final JList	jl=new JList(files);
			jl.setCellRenderer(this);
			jl.addMouseListener(new MouseAdapter() 
				{
	      			@Override
					public void mouseClicked(MouseEvent me) 
	      			{
	      				int inx=jl.locationToIndex(me.getPoint());
					
	      				if(inx>=0)
	      				{
	      					Rectangle	rect=jl.getCellBounds(inx, inx);
	      					int			x=me.getX()-rect.x;
	      					int			y=me.getY()-rect.y;
	      					
	      					rect=checkBox.getBounds();
	      					Insets	ins=checkBox.getInsets();
	      					if(ins!=null)
	      					{
	      						rect.x+=ins.left;
	      						rect.y+=ins.top;
	      						rect.width-=(ins.left+ins.right);
	      						rect.height-=(ins.top+ins.bottom);
	      					}
	      					
	      					if(rect.contains(x, y))
	      					{
		      					selFiles[inx]=!selFiles[inx];
		      					jl.repaint();
	      					}
	      				}
	      			}
	      		});
			
			add(new JScrollPane(jl, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), BorderLayout.CENTER);
		
			msgPanel.add(this, BorderLayout.CENTER);
			
			validate();		
			MessageDialog.this.pack();	
		}
		
		boolean[] getFilesSelection()
		{
			return selFiles;
		}

		@Override
		public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) 
		{
			File	f=files[index];
			defRenderer.getListCellRendererComponent (list, f.getName()+"  ["+f.getPath()+"]", index, isSelected, cellHasFocus);  //$NON-NLS-1$//$NON-NLS-2$

			checkBox.setSelected(selFiles[index]);
			
			cellComp.setOpaque(defRenderer.isOpaque());
			cellComp.setForeground(defRenderer.getForeground());
			cellComp.setBackground(defRenderer.getBackground());

			checkBox.setOpaque(defRenderer.isOpaque());
			checkBox.setForeground(defRenderer.getForeground());
			checkBox.setBackground(defRenderer.getBackground());
			
			return cellComp;
		}
	}
	
	class InputPanel extends JPanel implements DocumentListener, ActionListener
	{
		private JTextField	textField;
		
		InputPanel(String msg, String value, Component addComp)
		{
			super(new GridBagLayout());

			
			JPanel	p=new JPanel();
			p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
			
			JLabel	jl=new JLabel(msg);
			jl.setBorder(new EmptyBorder(4, 4, 4, 4));
			
			p.add(jl);
			textField=new JTextField(value);
			textField.getDocument().addDocumentListener(this);
			textField.addActionListener(this);
			p.add(textField);
			
			if(addComp!=null)
			{
				JPanel	p2=new JPanel(new BorderLayout(4,4));
				
				p2.add(p, BorderLayout.NORTH);
				p2.add(addComp, BorderLayout.SOUTH);
				
				add(p2);
			}
			else
				add(p);
			
			msgPanel.add(this, BorderLayout.CENTER);
			
			enableOKButton();
			
			validate();
			MessageDialog.this.pack();	
		}

		String getText()
		{
			return textField.getText();
		}
		
		private void enableOKButton()
		{
			buttons[0].setEnabled(textField.getText().length()>0);
		}
		
		@Override
		public void changedUpdate(DocumentEvent e) 
		{
			enableOKButton();
		}

		@Override
		public void insertUpdate(DocumentEvent e) 
		{
			enableOKButton();
		}

		@Override
		public void removeUpdate(DocumentEvent e) 
		{
			enableOKButton();
		}

		@Override
		public void actionPerformed(ActionEvent e) 
		{
			if(textField.getText().length()>0)
				optionActionPerformed((ActionOption)buttons[0].getAction());
		}
	}
}