/* JSGalBuilder: (c) 2009 by Andreas S. Krebs
 * 
 * This file is part of JSGalBuilder.
 *
 * JSGalBuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSGalBuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSGalBuilder.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sberk.jsgal.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.URI;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.Action;
import javax.swing.DropMode;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.TransferHandler;
import javax.swing.TransferHandler.DropLocation;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.event.DocumentListener;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

@SuppressWarnings("serial")
public class SwingHelper
{
	public static Border	DEFAULT_LABEL_BORDER=new EmptyBorder(0, 4, 0, 0);
	public static Border	DEFAULT_COMP_BORDER=new EmptyBorder(2, 0, 2, 0);
	
	public static JCheckBox addCheckBox(JComponent parent, String label, boolean value)
	{
		return addCheckBox(parent, label, value, null);
	}

	public static JCheckBox addCheckBox(JComponent parent, String label, boolean value, ActionListener l)
	{
		return addCheckBox(parent, label, value, l, true);
	}
	
	public static JCheckBox addCheckBox(JComponent parent, String label, boolean value, ActionListener l, boolean inNewPanel)
	{
		JCheckBox	jcb=new JCheckBox(label, value);
		
		if(l!=null)
			jcb.addActionListener(l);
		
		if(inNewPanel)
		{
			JPanel	p=new JPanel(new BorderLayout());
			p.add(jcb, BorderLayout.WEST);
			p.setBorder(DEFAULT_COMP_BORDER);
			
			parent.add(p);
		}
		else
			parent.add(jcb);
		
		return jcb;
	}
	
	public static JComboBox addComboBox(JComponent parent, String label, String values[], String value)
	{
		return addComboBox(parent, label, values, value, null);
	}

	public static JComboBox addComboBox(JComponent parent, String label, String values[], String value, ActionListener l)
	{
		JLabel		jlbl=new JLabel(label);
		jlbl.setBorder(DEFAULT_LABEL_BORDER);
		JComboBox	jcb=new JComboBox(values);
		jcb.setSelectedItem(value);
		
		if(l!=null)
			jcb.addActionListener(l);
		
		JPanel	p1=new JPanel(new BorderLayout(4, 4));

		p1.add(jlbl, BorderLayout.WEST);
		p1.add(jcb, BorderLayout.CENTER);
		JPanel	p2=new JPanel(new BorderLayout());
		p2.add(p1, BorderLayout.WEST);
		p2.setBorder(DEFAULT_COMP_BORDER);

		parent.add(p2);
		
		return jcb;
	}

	public static JTextField addIntegerField(JComponent parent, String label, int length, int value, DocumentListener l)
	{
		return addIntegerField(parent, label, length, value, l, true);
	}
	
	public static JTextField addIntegerField(JComponent parent, String label, int length, int value, DocumentListener l, boolean inNewPanel)
	{
		JTextField	jtf=addTextField(parent, label, length, null, null, inNewPanel);
		
		jtf.setDocument(new IntegerTextDocument());
		jtf.setText(Integer.toString(value));
		
		if(l!=null)
			jtf.getDocument().addDocumentListener(l);
		
		return jtf;
	}

	public static int getIntValue(JTextField jtf)
	{
		try 
		{
			return Integer.parseInt(jtf.getText());	
		}
		catch(NumberFormatException e) 
		{
			return 0;
		}
	}
	
	public static JTextField addFloatField(JComponent parent, String label, int length, float value, DocumentListener l)
	{
		JTextField	jtf=addTextField(parent, label, length, null, null, true, null);

		jtf.setDocument(new FloatTextDocument());
		jtf.setText(Float.toString(value));

		if(l!=null)
			jtf.getDocument().addDocumentListener(l);
		
		return jtf;
	}
	
	public static float getFloatValue(JTextField jtf)
	{
		try 
		{
			return Float.parseFloat(jtf.getText());	
		}
		catch(NumberFormatException e) 
		{
			return 0.0f;
		}
	}

	public static JTextField addTextField(JComponent parent, String label, int length, String value)
	{
		return addTextField(parent, label, length, value, null, true, null);
	}
	
	public static JTextField addTextField(JComponent parent, String label, int length, String value, DocumentListener l)
	{
		return addTextField(parent, label, length, value, l, true, null);
	}

	public static JTextField addTextField(JComponent parent, String label, int length, String value, DocumentListener l, Component addComp)
	{
		return addTextField(parent, label, length, value, l, true, addComp);
	}
	
	public static JTextField addTextField(JComponent parent, String label, int length, String value, DocumentListener l, boolean inNewPanel)
	{
		return addTextField(parent, label, length, value, l, inNewPanel, null);
	}
	
	private static JTextField addTextField(JComponent parent, String label, int length, String value, DocumentListener l, boolean inNewPanel, Component addComp)
	{
		JLabel		jlbl=new JLabel(label);
		JTextField	jtf=new JTextField(value, length);
		jlbl.setLabelFor(jtf);
		
		if(l!=null)
			jtf.getDocument().addDocumentListener(l);
		
		if(inNewPanel || addComp!=null)
		{
			jlbl.setBorder(DEFAULT_LABEL_BORDER);
			
			JPanel	p1=new JPanel(new BorderLayout(4, 4));
			p1.add(jlbl, BorderLayout.WEST);
			p1.add(jtf, BorderLayout.CENTER);
			if(addComp!=null)
				p1.add(addComp, BorderLayout.EAST);
			
			JPanel	p2=new JPanel(new BorderLayout());
			p2.add(p1, BorderLayout.WEST);
			p2.setBorder(DEFAULT_COMP_BORDER);
			
			parent.add(p2);
		}
		else
		{
			parent.add(jlbl);
			parent.add(jtf);
		}
		
		return jtf;
	}
	
	static abstract class NumberTextDocument extends PlainDocument 
	{
		abstract boolean isValid(String s);
		
		@Override
		public void insertString(int offs, String str, AttributeSet a) throws BadLocationException
		{
			if(str!=null)
			{
				String	os=getText(0, getLength());
				String	ns=os.substring(0, offs)+str+os.substring(offs);
		
				if(isValid(ns))
					super.insertString(offs, str, a);
			}
		}
	}	

	static class IntegerTextDocument extends NumberTextDocument 
	{
		@Override
		boolean isValid(String s)
		{
			try 
			{
				if(s.length()<=0)	// empty string is valid
					return true;	
				else if(s.length()==1)	// special case for sign 
				{
					char	c=s.charAt(0);
					
					if(c=='+' || c=='-')
						return true;
				}
				
				Integer.parseInt(s);
				return true;
			}
			catch(NumberFormatException e) 
			{
				// we ignore this insert
			}
			
			return false;
		}
	}	

	static class FloatTextDocument extends NumberTextDocument 
	{
		@Override
		boolean isValid(String s)
		{
			try 
			{
				if(s.length()<=0)	// empty string is valid
					return true;	
				else if(s.length()==1)	// special case for sign 
				{
					char	c=s.charAt(0);
					
					if(c=='+' || c=='-' || c=='.')
						return true;
				}
				
				Float.parseFloat(s);
				return true;
			}
			catch(NumberFormatException e) 
			{
				// we ignore this insert
			}
			
			return false;
		}
	}	
	
	public static void showPopup(Component parent, int x, int y, Collection<Action> col)
	{
		if(col!=null && col.size()>0)
		{
			JPopupMenu	jpm=new JPopupMenu();
			
			for(Action a: col)
			{
				if(a!=null)
					jpm.add(a);
				else
					jpm.addSeparator();
			}
			
			jpm.show(parent, x, y);
		}
	}

	public static void enableFileDrop(JTable jt, FileDropListener listener)
	{
		jt.setTransferHandler(new FileTransferHandler(listener));
		jt.setDropMode(DropMode.ON);
		jt.setDragEnabled(true);
	}
	
	public static void enableFileDrop(JList jl, FileDropListener listener)
	{
		jl.setTransferHandler(new FileTransferHandler(listener));
		jl.setDropMode(DropMode.ON);
		jl.setDragEnabled(true);
	}	
	
	public static void enableFileDrop(JTree jt, FileDropListener listener)
	{
		jt.setTransferHandler(new FileTransferHandler(listener));
		jt.setDropMode(DropMode.ON);
		jt.setDragEnabled(true);
	}
	
	public static interface FileDropListener
	{
		public boolean canImport(DropLocation location);
		public void filesDropped(DropLocation location, File files[]);
	}
}

@SuppressWarnings("serial")
class FileTransferHandler extends TransferHandler
{
    private static final String URI_LIST_MIME_TYPE	="text/uri-list;class=java.lang.String"; //$NON-NLS-1$

    private DataFlavor						uriListFlavor;
	private SwingHelper.FileDropListener	listener;
	
	FileTransferHandler(SwingHelper.FileDropListener listener)
	{
		this.listener=listener;
		
		try
		{
			this.uriListFlavor=new DataFlavor(URI_LIST_MIME_TYPE);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	
	@Override
	public int getSourceActions(JComponent c) 
	{
	    return COPY;
	}
	 
	@Override
	public boolean canImport(TransferSupport support) 
	{
		for(DataFlavor f: support.getDataFlavors())
		{
			if(f.equals(DataFlavor.javaFileListFlavor) || f.equals(uriListFlavor))
				return listener.canImport(support.getDropLocation());
		}

		return false;
	}		
	
	@Override
	public boolean importData(TransferSupport support) 
	{
		Transferable	t=support.getTransferable();
		
		for(DataFlavor f: t.getTransferDataFlavors())
		{
			try
			{
				List<?>	fl;
				 
				if(f.equals(DataFlavor.javaFileListFlavor))	// win
				{
					fl=(List<?>)t.getTransferData(DataFlavor.javaFileListFlavor);		
				}
				else if(f.equals(uriListFlavor))	// linux
				{
					String	s=(String)t.getTransferData(uriListFlavor);
					 
					fl=uri2fileList(s);
				}
				else
					continue;
				
				if(!listener.canImport(support.getDropLocation()))
					return false;
				
				listener.filesDropped(support.getDropLocation(), fl.toArray(new File[fl.size()]));
				return true;
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
		 
		return false;
	}
	
	private List<File> uri2fileList(String urilist)
	{
		List<File> l=new ArrayList<File>();
		
		String	sa[]=urilist.split("\r\n|\n"); //$NON-NLS-1$
		for(String s: sa)
		{
			if(s.startsWith("#"))	// comment //$NON-NLS-1$
				continue;

			try
			{
				l.add(new File(new URI(s)));
			}
			catch(Exception e) 
			{
				e.printStackTrace();
			}
	    }
	    
		return l;
	}
}