/* JSGalBuilder: (c) 2009 by Andreas S. Krebs
 * 
 * This file is part of JSGalBuilder.
 *
 * JSGalBuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSGalBuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSGalBuilder.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sberk.util;

import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import com.drew.imaging.jpeg.JpegMetadataReader;
import com.drew.metadata.Directory;
import com.drew.metadata.Metadata;
import com.drew.metadata.exif.ExifDirectory;

public class ExifExtractor
{
	private final static DateFormat	DATE_FORMAT=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"); //$NON-NLS-1$
	
	public static ExifMetaData getMetaData(File f)
	{
		String		dateTime=null;
		Exception	exifEx=null;
		int			orient=0;
		
		try
		{
			Metadata	md=JpegMetadataReader.readMetadata(f);
			
			Directory	exifDir=md.getDirectory(ExifDirectory.class);
		
			// now get the md we're interested in
			dateTime=exifDir.getString(ExifDirectory.TAG_DATETIME);
			orient=exifDir.getInt(ExifDirectory.TAG_ORIENTATION);
			
			// write all tags to stdout
//			Iterator<?>	dirs=md.getDirectoryIterator();
//			while(dirs.hasNext())
//			{
//				Iterator<?>	tags=((Directory)dirs.next()).getTagIterator();
//				while(tags.hasNext())
//				{
//					Tag	tag=(Tag)tags.next();
//					
//					System.out.println(tag);
//				}
//			}
		}
		catch(Exception e)
		{
			exifEx=e;
		}
		
		if(dateTime==null || dateTime.length()<=0)	// no exif, so let's get the file date time
		{
			long	l=f.lastModified();
			
			dateTime=DATE_FORMAT.format(new Date(l));
		}
		
		return new ExifMetaData(exifEx, dateTime, f.getName(), orient);
	}
	
	// a small utility class only containing what we're interested in
	public static class ExifMetaData
	{
		public Exception	ExifException;
		public String		DateTime;
		public String		FileName;
		public int			Orientation;
		
		public ExifMetaData(Exception ex, String dt, String fn, int orient)
		{
			this.ExifException=ex;
			this.DateTime=dt;
			this.FileName=fn;
			this.Orientation=orient;
		}
	}
}