/* JSGalBuilder: (c) 2009 by Andreas S. Krebs
 * 
 * This file is part of JSGalBuilder.
 *
 * JSGalBuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSGalBuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSGalBuilder.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sberk.util;

import java.io.*;

import javax.xml.parsers.*;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * A helper class for parsing and writing XML.
 * 
 * @author Andreas S. Krebs
 */
public class XMLUtil
{
	public static Document parseXML(File f, boolean validating)
			throws SAXException, ParserConfigurationException, IOException
	{
		return createBuilder(validating).parse(f);
	}

	public static Document parseXML(InputStream is, boolean validating)
		throws SAXException, ParserConfigurationException, IOException
	{
		return createBuilder(validating).parse(is);
	}
	
	public static Document createDOM() throws ParserConfigurationException
	{
		return createBuilder(false).newDocument();
	}	
	
	private static DocumentBuilder createBuilder(boolean validating) throws ParserConfigurationException
	{
		DocumentBuilderFactory factory=DocumentBuilderFactory.newInstance();

		factory.setValidating(validating);
		
		DocumentBuilder	db=factory.newDocumentBuilder();

		if(!validating)
		{
			// avoid stupid DTD resolving if not validating !!!
			db.setEntityResolver(new EntityResolver() {
		        @Override
		        public InputSource resolveEntity(String publicId, String systemId) throws SAXException, IOException
		        {
		        	// simply return an empty input source for everything !!!
	                return new InputSource(new StringReader("")); //$NON-NLS-1$
		        }
		    });
		}		
		
		return db;
	}
	
	public static void writeXML(Document doc, OutputStream os)
			throws TransformerConfigurationException, TransformerException, IOException
	{
		writeXML(doc, true, false, null, null, os);
	}
	
	public static void writeXML(Document doc, String publicId, String systemId, OutputStream os)
			throws TransformerConfigurationException, TransformerException, IOException
	{
		writeXML(doc, true, false, publicId, systemId, os);
	}
	
	@SuppressWarnings("nls")
	public static void writeXML(Document doc, boolean indent, boolean omitXMLDecl, String publicId, String systemId, OutputStream os)
			throws TransformerConfigurationException, TransformerException, IOException
	{
		Transformer xformer=TransformerFactory.newInstance().newTransformer();

		if(publicId!=null)
			xformer.setOutputProperty(OutputKeys.DOCTYPE_PUBLIC, publicId);

		if(systemId!=null)
			xformer.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM, systemId);

		// set output properties
		xformer.setOutputProperty(OutputKeys.METHOD, "xml");
		xformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, omitXMLDecl ? "yes" : "no");
		
		if(indent)
		{
			// for xml we do indent !!!
			xformer.setOutputProperty(OutputKeys.INDENT, "yes");
			
			xformer.setOutputProperty("{http://xml.apache.org/xalan}indent-amount", "2");
			xformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
		}
	      
		xformer.transform(new DOMSource(doc), new StreamResult(os));
		
		// finally close the stream - just to be save
		os.close();
	}
	
	public static String getElementText(Element el)
	{
		StringBuffer	sb=new StringBuffer();
		NodeList		nl=el.getChildNodes();
		
		for(int i=0; i<nl.getLength(); i++)
		{
			Node	nd=nl.item(i);
			
			if(nd instanceof Text)
				sb.append(((Text)nd).getNodeValue());
		}
		
		return sb.toString();
	}
}
